@@ -55,7 +55,7 @@ class ReplaceMatcher:
         forms_lookup=None,
         custom_match_hooks: Optional[ModuleType] = None,
         allow_multiple_whitespaces=False,
-        lemmatizer="pyInflect"
+        lemmatizer="pyInflect",
     ):
         self.default_match_hooks = default_match_hooks
         self.custom_match_hooks = custom_match_hooks
@@ -67,7 +67,9 @@ class ReplaceMatcher:
         self.matcher = Matcher(self.nlp.vocab)
         self._init_matcher()
         self.spans: List[Span] = []
-        self.inflector = Inflector(nlp=self.nlp, forms_lookup=self.forms_lookup, lemmatizer=lemmatizer)
+        self.inflector = Inflector(
+            nlp=self.nlp, forms_lookup=self.forms_lookup, lemmatizer=lemmatizer
+        )
 
         # set custom extensions for any unexpected keys found in the match_dict
         novel_properites = (
@@ -120,12 +122,23 @@ class ReplaceMatcher:
             # predicate - filled template ex. succeeded_by_word("to")
             # will match "in addition to..." but not "in addition, ..."
             args = hook.get("args", None)
-            if args is not None:
-                # the match_hook needs arguments
-                pred = template(hook["args"])
+            kwargs = hook.get("kwargs", None)
+            if args is None:
+                if kwargs is None:
+                    # the match_hook is nullary
+                    pred = template()
+                else:
+                    pred = template(**kwargs)
+            elif type(args) == dict:
+                # should we force them to use kwargs?
+                print(
+                    f"WARNING: dict passed as sole args argument. Calling {hook['name']} "
+                    f"with single argument {args}. If you want to call with keyword arguments, use kwargs"
+                )
+                pred = template(args)
             else:
-                # the match_hook is nullary
-                pred = template()
+                # oops, bad design, we assume non-dicts are called directly
+                pred = template(args)
 
             # to confuse people for centuries to come ...
             # negate, since positive breaks matching
@@ -136,8 +149,9 @@ class ReplaceMatcher:
             predicates.append(pred)
         return predicates
 
-    def inflect_suggestion(self, pre_suggestion, doc, start, end, match_name):
+    def process_suggestions(self, pre_suggestion, doc, start, end, match_name):
         """
+        Perform inflection and replace references to the matched token
         example of pattern and pre_suggestion
         pattern: "LEMMA": "chock", "TEMPLATE_ID": 1
         pre_suggestion: "TEXT": "chalk", "FROM_TEMPLATE_ID": 1
@@ -145,7 +159,11 @@ class ReplaceMatcher:
         """
         text_list = []
         for item in pre_suggestion:
-            text = item["TEXT"]
+            try:
+                text = item["TEXT"]
+            except KeyError:
+                ref = int(item["PATTERN_REF"])
+                refd_token = doc[start + ref]
             changed_text = None
 
             # check if inflect
@@ -157,7 +175,13 @@ class ReplaceMatcher:
                         index = i
                         break
                 if index is not None:
-                    changed_text = self.inflector.inflect(doc, text, start + index)
+                    changed_text = self.inflector.auto_inflect(doc, text, start + index)
+            elif "PATTERN_REF" in item:
+                if "INFLECTION" in item:
+                    form = item["INFLECTION"]
+                    changed_text = self.inflector.inflect_or_lookup(refd_token, form)
+                else:
+                    changed_text = refd_token.text
             if changed_text:
                 text_list.append(changed_text)
             elif len(text):
@@ -189,11 +213,11 @@ class ReplaceMatcher:
 
             pre_suggestions = self.match_dict[match_name]["suggestions"]
 
-            span._.suggestions = (
-                seq(pre_suggestions)
-                .map(lambda x: self.inflect_suggestion(x, doc, start, end, match_name))
-                .list()
-            )
+            span._.suggestions = [
+                self.process_suggestions(x, doc, start, end, match_name)
+                for x in pre_suggestions
+            ]
+
             span._.description = self.match_dict[match_name].get("description", "")
             span._.category = self.match_dict[match_name].get("category", "")
             for novel_prop, default_value in self.novel_prop_defaults.items():
